// PXEWIN - (C) Copyright 1992 by Beam Engineering, INC.

// PXEOBJ.CPP //

// Contents ----------------------------------------------------------------
//
//	This module contains class members for the PXEngObject class.
//
// End ---------------------------------------------------------------------

// External Reference Name For This Header ---------------------------------

#ifndef PXEOBJ_CPP
	#define PXEOBJ_CPP

// End ---------------------------------------------------------------------

// Interface Dependencies --------------------------------------------------

#ifndef PXEOBJ_HPP
	#include "pxeobj.hpp"
#endif // PXEOBJ_HPP //

// End ---------------------------------------------------------------------

// Constructor PXEngObject //

inline PXEngObject::PXEngObject()
{
	close_status = CLOSED;
}

// Description -------------------------------------------------------------
//
//	Initialize error status to no errors.
//
// End ---------------------------------------------------------------------

// member PXError of PXEngObject //

void PXEngObject::PXError(int org)
{
	// Copy origin ot Error data

	EngDataPtr->Errors.Origin = org;

	// Switch in correct member for return string

	switch(EngDataPtr->Errors.Origin)
	{
		case ENG_ERROR:
		{
			EngDataPtr->Errors.EngMem = &PXEngObject::RetPXMsg;
			break;
		}
		case EWIN_ERROR:
		{
			EngDataPtr->Errors.EngMem =
				&PXEngObject::RetPXEWINErrorMsg;
			break;
		}
	}
}

// Summary -----------------------------------------------------------------
//
//	This member will switch in the correct return error string function
//	depending on the origin of the error.
//
// End ---------------------------------------------------------------------

// member build of PXEngObject //

PTStreamable PXEngObject::build()
{
	return new PXEngObject(streamableInit);
}

TStreamableClass RegPXEngObject("PXEngObject",PXEngObject::build,
	__DELTA(PXEngObject));

// Description -------------------------------------------------------------
//
//	When the streamable constructor is called, TStreamable dispatches
//	the build member to construct the object.  To do this, it must
//	know where to find this member functions for the specific class.
//	This is the reason for the stream registration.
//
// End ---------------------------------------------------------------------

// member read of PXEngObject //

inline Pvoid PXEngObject::read(Ripstream)
{
	close_status = CLOSED;
	return this;
}

// Description -------------------------------------------------------------
//
//	Since read is called after the streamable constructor is called, you
//	need to set the PXEngObject to it's default state with no errors.
//	Nothing is taken off the stream.  You could have put this with the
//	streamable constructor but by convension I have put it here.
//
// End ---------------------------------------------------------------------

// member write of PXEngObject //

inline void PXEngObject::write(Ropstream)
{

}

// Description -------------------------------------------------------------
//
//	Nothing is streamed here.  This is here only to make an instance of
//	this class.
//
// End ---------------------------------------------------------------------

#endif // PXEOBJ_CPP //

